"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const styles_1 = require("../styles");
module.exports = {
    meta: {
        docs: {
            description: "Enforce the `<style>` tags to has the `scoped` attribute.",
            category: "recommended",
            default: "warn",
            url: "https://future-architect.github.io/eslint-plugin-vue-scoped-css/rules/require-scoped.html",
        },
        fixable: null,
        messages: {
            missing: "Missing `scoped` attribute.",
        },
        schema: [],
        type: "suggestion",
    },
    create(context) {
        const styles = styles_1.getStyleContexts(context).filter(style => !style.invalid);
        if (!styles.length) {
            return {};
        }
        const reporter = styles_1.getCommentDirectivesReporter(context);
        function report(node) {
            reporter.report({
                node: node.startTag,
                messageId: "missing",
                data: {},
            });
        }
        return {
            "Program:exit"() {
                for (const style of styles) {
                    if (!style.scoped) {
                        report(style.styleElement);
                    }
                }
            },
        };
    },
};
