"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class Node {
    constructor(node, type, loc, start, end, lang) {
        this.type = type;
        this.loc = loc;
        this.start = start;
        this.end = end;
        this.range = [start, end];
        this.node = node;
        this.lang = lang;
    }
}
class HasParentNode extends Node {
    constructor(node, type, loc, start, end, props) {
        super(node, type, loc, start, end, props.parent.lang);
        this.parent = props.parent;
    }
}
class VCSSParsingError extends Node {
    constructor(node, loc, start, end, props) {
        super(node, "VCSSParsingError", loc, start, end, props.lang);
        this.message = props.message;
    }
}
exports.VCSSParsingError = VCSSParsingError;
class VCSSStyleSheet extends Node {
    constructor(node, loc, start, end, props) {
        var _a, _b, _c;
        super(node, "VCSSStyleSheet", loc, start, end, props.lang);
        this.nodes = (_a = props.nodes, (_a !== null && _a !== void 0 ? _a : []));
        this.comments = (_b = props.comments, (_b !== null && _b !== void 0 ? _b : []));
        this.errors = (_c = props.errors, (_c !== null && _c !== void 0 ? _c : []));
    }
    copy(props) {
        return copyStdNode(this, props);
    }
}
exports.VCSSStyleSheet = VCSSStyleSheet;
class VCSSStyleRule extends HasParentNode {
    constructor(node, loc, start, end, props) {
        var _a, _b, _c;
        super(node, "VCSSStyleRule", loc, start, end, props);
        this.selectorText = (_a = props.selectorText, (_a !== null && _a !== void 0 ? _a : node.selector));
        if (props.rawSelectorText != null) {
            this.rawSelectorText = props.rawSelectorText;
        }
        else {
            const raws = node.raws;
            this.rawSelectorText = raws.selector
                ? raws.selector.raw
                : node.selector;
        }
        this.selectors = (_b = props.selectors, (_b !== null && _b !== void 0 ? _b : []));
        this.nodes = (_c = props.nodes, (_c !== null && _c !== void 0 ? _c : []));
    }
    copy(props) {
        return copyStdNode(this, props);
    }
}
exports.VCSSStyleRule = VCSSStyleRule;
class VCSSDeclarationProperty extends HasParentNode {
    constructor(node, loc, start, end, props) {
        var _a, _b;
        super(node, "VCSSDeclarationProperty", loc, start, end, props);
        this.property = (_a = props.property, (_a !== null && _a !== void 0 ? _a : node.prop));
        this.value = getProp(props, node, "value");
        this.important = (_b = props.important, (_b !== null && _b !== void 0 ? _b : Boolean(node.important)));
    }
    copy(props) {
        return copyStdNode(this, props);
    }
}
exports.VCSSDeclarationProperty = VCSSDeclarationProperty;
class VCSSAtRule extends HasParentNode {
    constructor(node, loc, start, end, props) {
        var _a, _b, _c, _d, _e;
        super(node, "VCSSAtRule", loc, start, end, props);
        this.node = node;
        this.name = getProp(props, node, "name");
        this.paramsText = (_a = props.paramsText, (_a !== null && _a !== void 0 ? _a : node.params));
        if (props.rawParamsText != null) {
            this.rawParamsText = props.rawParamsText;
        }
        else {
            const raws = node.raws;
            this.rawParamsText = (_c = (_b = raws.params) === null || _b === void 0 ? void 0 : _b.raw, (_c !== null && _c !== void 0 ? _c : node.params));
        }
        this.selectors = (_d = props.selectors, (_d !== null && _d !== void 0 ? _d : undefined));
        this.nodes = (_e = props.nodes, (_e !== null && _e !== void 0 ? _e : []));
    }
    copy(props) {
        return copyStdNode(this, props);
    }
}
exports.VCSSAtRule = VCSSAtRule;
class VCSSUnknown extends HasParentNode {
    constructor(node, loc, start, end, props) {
        var _a;
        super(node, "VCSSUnknown", loc, start, end, props);
        this.nodes = (_a = props.nodes, (_a !== null && _a !== void 0 ? _a : []));
        this.unknownType = props.unknownType;
    }
    copy(props) {
        return copyStdNode(this, props);
    }
}
exports.VCSSUnknown = VCSSUnknown;
class VCSSSelector extends HasParentNode {
    constructor(node, loc, start, end, props) {
        var _a;
        super(node, "VCSSSelector", loc, start, end, props);
        this.nodes = (_a = props.nodes, (_a !== null && _a !== void 0 ? _a : []));
        this.parent = props.parent;
    }
    copy(props) {
        return copyStdNode(this, props);
    }
    get selector() {
        return this.nodes.map(n => n.selector).join("");
    }
}
exports.VCSSSelector = VCSSSelector;
class VCSSTypeSelector extends HasParentNode {
    constructor(node, loc, start, end, props) {
        super(node, "VCSSTypeSelector", loc, start, end, props);
        this.value = getProp(props, node, "value");
        this.selector = this.value;
    }
    copy(props) {
        return copyStdNode(this, props);
    }
}
exports.VCSSTypeSelector = VCSSTypeSelector;
class VCSSIDSelector extends HasParentNode {
    constructor(node, loc, start, end, props) {
        super(node, "VCSSIDSelector", loc, start, end, props);
        this.value = getProp(props, node, "value");
        this.selector = `#${this.value}`;
    }
    copy(props) {
        return copyStdNode(this, props);
    }
}
exports.VCSSIDSelector = VCSSIDSelector;
class VCSSClassSelector extends HasParentNode {
    constructor(node, loc, start, end, props) {
        super(node, "VCSSClassSelector", loc, start, end, props);
        this.value = getProp(props, node, "value");
        this.selector = `.${this.value}`;
    }
    copy(props) {
        return copyStdNode(this, props);
    }
}
exports.VCSSClassSelector = VCSSClassSelector;
class VCSSNestingSelector extends HasParentNode {
    constructor(node, loc, start, end, props) {
        super(node, "VCSSNestingSelector", loc, start, end, props);
        this.value = getProp(props, node, "value");
        this.selector = this.value;
    }
    copy(props) {
        return copyStdNode(this, props);
    }
}
exports.VCSSNestingSelector = VCSSNestingSelector;
class VCSSUniversalSelector extends HasParentNode {
    constructor(node, loc, start, end, props) {
        super(node, "VCSSUniversalSelector", loc, start, end, props);
        this.value = getProp(props, node, "value");
        this.selector = this.value;
    }
    copy(props) {
        return copyStdNode(this, props);
    }
}
exports.VCSSUniversalSelector = VCSSUniversalSelector;
class VCSSAttributeSelector extends HasParentNode {
    constructor(node, loc, start, end, props) {
        super(node, "VCSSAttributeSelector", loc, start, end, props);
        this.attribute = getProp(props, node, "attribute");
        const operator = getProp(props, node, "operator");
        this.operator = (operator !== null && operator !== void 0 ? operator : null);
        const value = getProp(props, node, "value");
        this.value = (value !== null && value !== void 0 ? value : null);
        const quoteMark = getProp(props, node, "quoteMark");
        this.quoteMark = (quoteMark !== null && quoteMark !== void 0 ? quoteMark : null);
        const raws = node.raws;
        if (props.insensitiveFlag != null) {
            this.insensitiveFlag = props.insensitiveFlag;
        }
        else if (raws.insensitiveFlag != null) {
            this.insensitiveFlag = raws.insensitiveFlag;
        }
        else if (node.insensitive) {
            this.insensitiveFlag = "i";
        }
        else {
            this.insensitiveFlag = null;
        }
        this.selector = this.refreshSelector();
    }
    refreshSelector() {
        let selector = `[${this.attribute}`;
        if (this.operator != null) {
            selector += this.operator;
            if (this.value != null) {
                selector += this.quoteMark
                    ? this.quoteMark + this.value + this.quoteMark
                    : this.value;
            }
        }
        if (this.insensitiveFlag != null) {
            selector += ` ${this.insensitiveFlag}`;
        }
        selector += "]";
        return selector;
    }
    copy(props) {
        return copyStdNode(this, props);
    }
}
exports.VCSSAttributeSelector = VCSSAttributeSelector;
class VCSSSelectorPseudo extends HasParentNode {
    constructor(node, loc, start, end, props) {
        var _a;
        super(node, "VCSSSelectorPseudo", loc, start, end, props);
        this.value = getProp(props, node, "value");
        this.nodes = (_a = props.nodes, (_a !== null && _a !== void 0 ? _a : []));
    }
    copy(props) {
        return copyStdNode(this, props);
    }
    get selector() {
        if (!this.nodes.length) {
            return this.value;
        }
        const params = this.nodes.map(n => n.selector).join(",");
        return `${this.value}(${params})`;
    }
}
exports.VCSSSelectorPseudo = VCSSSelectorPseudo;
class VCSSSelectorCombinator extends HasParentNode {
    constructor(node, loc, start, end, props) {
        super(node, "VCSSSelectorCombinator", loc, start, end, props);
        this.value = getProp(props, node, "value");
        this.selector = this.value;
    }
    copy(props) {
        return copyStdNode(this, props);
    }
}
exports.VCSSSelectorCombinator = VCSSSelectorCombinator;
class VCSSUnknownSelector extends HasParentNode {
    constructor(node, loc, start, end, props) {
        super(node, "VCSSUnknownSelector", loc, start, end, props);
        this.value = getProp(props, node, "value") || "";
        this.selector = this.value;
    }
    copy(props) {
        return copyStdNode(this, props);
    }
}
exports.VCSSUnknownSelector = VCSSUnknownSelector;
class VCSSComment extends HasParentNode {
    constructor(node, text, loc, start, end, props) {
        super(node, "VCSSComment", loc, start, end, props);
        this.node = node;
        this.text = text;
    }
    copy(props) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m;
        const parent = (_b = (_a = props) === null || _a === void 0 ? void 0 : _a.parent, (_b !== null && _b !== void 0 ? _b : this.parent));
        return new VCSSComment((_d = (_c = props) === null || _c === void 0 ? void 0 : _c.node, (_d !== null && _d !== void 0 ? _d : this.node)), (_f = (_e = props) === null || _e === void 0 ? void 0 : _e.text, (_f !== null && _f !== void 0 ? _f : this.text)), (_h = (_g = props) === null || _g === void 0 ? void 0 : _g.loc, (_h !== null && _h !== void 0 ? _h : this.loc)), (_k = (_j = props) === null || _j === void 0 ? void 0 : _j.start, (_k !== null && _k !== void 0 ? _k : this.start)), (_m = (_l = props) === null || _l === void 0 ? void 0 : _l.end, (_m !== null && _m !== void 0 ? _m : this.end)), Object.assign(Object.assign(Object.assign({}, this), props), { parent }));
    }
}
exports.VCSSComment = VCSSComment;
class VCSSInlineComment extends HasParentNode {
    constructor(node, text, loc, start, end, props) {
        super(node, "VCSSInlineComment", loc, start, end, props);
        this.node = node;
        this.text = text;
    }
    copy(props) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m;
        const parent = (_b = (_a = props) === null || _a === void 0 ? void 0 : _a.parent, (_b !== null && _b !== void 0 ? _b : this.parent));
        return new VCSSInlineComment((_d = (_c = props) === null || _c === void 0 ? void 0 : _c.node, (_d !== null && _d !== void 0 ? _d : this.node)), (_f = (_e = props) === null || _e === void 0 ? void 0 : _e.text, (_f !== null && _f !== void 0 ? _f : this.text)), (_h = (_g = props) === null || _g === void 0 ? void 0 : _g.loc, (_h !== null && _h !== void 0 ? _h : this.loc)), (_k = (_j = props) === null || _j === void 0 ? void 0 : _j.start, (_k !== null && _k !== void 0 ? _k : this.start)), (_m = (_l = props) === null || _l === void 0 ? void 0 : _l.end, (_m !== null && _m !== void 0 ? _m : this.end)), Object.assign(Object.assign(Object.assign({}, this), props), { parent }));
    }
}
exports.VCSSInlineComment = VCSSInlineComment;
function getProp(props, node, name) {
    var _a;
    if (((_a = props) === null || _a === void 0 ? void 0 : _a[name]) != null) {
        const v = props[name];
        return v;
    }
    return node[name];
}
function copyStdNode(astNode, props) {
    var _a, _b, _c, _d, _e, _f, _g, _h;
    const C = astNode.constructor;
    return new C((_b = (_a = props) === null || _a === void 0 ? void 0 : _a.node, (_b !== null && _b !== void 0 ? _b : astNode.node)), (_d = (_c = props) === null || _c === void 0 ? void 0 : _c.loc, (_d !== null && _d !== void 0 ? _d : astNode.loc)), (_f = (_e = props) === null || _e === void 0 ? void 0 : _e.start, (_f !== null && _f !== void 0 ? _f : astNode.start)), (_h = (_g = props) === null || _g === void 0 ? void 0 : _g.end, (_h !== null && _h !== void 0 ? _h : astNode.end)), Object.assign(Object.assign({}, astNode), props));
}
