"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const style_1 = require("./style");
exports.StyleContext = style_1.StyleContext;
const comment_directive_1 = require("./comment-directive");
exports.CommentDirectivesReporter = comment_directive_1.CommentDirectivesReporter;
const vue_components_1 = require("./vue-components");
exports.VueComponentContext = vue_components_1.VueComponentContext;
const CACHE = new WeakMap();
function getCache(context) {
    const sourceCode = context.getSourceCode();
    const { ast } = sourceCode;
    if (CACHE.has(ast)) {
        return CACHE.get(ast);
    }
    const cache = {};
    CACHE.set(ast, cache);
    return cache;
}
function getStyleContexts(context) {
    const cache = getCache(context);
    if (cache.styles) {
        return cache.styles;
    }
    return (cache.styles = style_1.createStyleContexts(context));
}
exports.getStyleContexts = getStyleContexts;
function getCommentDirectivesReporter(context) {
    return comment_directive_1.createCommentDirectivesReporter(context, getCommentDirectives(context));
}
exports.getCommentDirectivesReporter = getCommentDirectivesReporter;
function getVueComponentContext(context) {
    const cache = getCache(context);
    if (cache.vueComponent) {
        return cache.vueComponent;
    }
    return (cache.vueComponent = vue_components_1.createVueComponentContext(context));
}
exports.getVueComponentContext = getVueComponentContext;
function getCommentDirectives(context) {
    const cache = getCache(context);
    if (cache.comment) {
        return cache.comment;
    }
    return (cache.comment = comment_directive_1.createCommentDirectives(getStyleContexts(context)));
}
