"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const postcss_scss_1 = __importDefault(require("postcss-scss"));
const css_parser_1 = require("./css-parser");
const ast_1 = require("../ast");
const scss_selector_parser_1 = require("./selector/scss-selector-parser");
class SCSSParser extends css_parser_1.CSSParser {
    parseInternal(css) {
        return postcss_scss_1.default.parse(css);
    }
    createSelectorParser() {
        return new scss_selector_parser_1.SCSSSelectorParser(this.sourceCode, this.commentContainer);
    }
    convertRuleNode(node, loc, start, end, parent) {
        var _a;
        let rawSelectorText = undefined;
        const rawsSelector = node.raws.selector;
        if (rawsSelector) {
            rawSelectorText = rawsSelector.scss;
        }
        const astNode = new ast_1.VCSSStyleRule(node, loc, start, end, {
            parent,
            rawSelectorText,
        });
        astNode.selectors = this.selectorParser.parse(astNode.rawSelectorText, astNode.loc.start, astNode);
        if ((_a = node.raws.between) === null || _a === void 0 ? void 0 : _a.trim()) {
            this.parseRuleRawsBetween(node, astNode);
        }
        return astNode;
    }
    convertCommentNode(node, loc, start, end, parent) {
        var _a;
        if ((_a = node.raws) === null || _a === void 0 ? void 0 : _a.inline) {
            this.commentContainer.push(new ast_1.VCSSInlineComment(node, node.text, loc, start, end, {
                parent,
            }));
        }
        else {
            super.convertCommentNode(node, loc, start, end, parent);
        }
        return null;
    }
}
exports.SCSSParser = SCSSParser;
