"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const selectors_1 = require("../../utils/selectors");
const ast_1 = require("../../ast");
const css_selector_resolver_1 = require("./css-selector-resolver");
exports.ResolvedSelector = css_selector_resolver_1.ResolvedSelector;
class SCSSSelectorResolver extends css_selector_resolver_1.CSSSelectorResolver {
    resolveNestingSelectors(selectorNodes, parentSelectors, container) {
        if (selectors_1.isNestingAtRule(container)) {
            return this.resolveSelectorForNestContaining(selectorNodes, parentSelectors, container);
        }
        return this.resolveSelectorForSCSSNesting(selectorNodes, parentSelectors, container);
    }
    resolveSelectorForSCSSNesting(selectorNodes, parentSelectors, container) {
        const nestings = selectors_1.findNestingSelectors(selectorNodes);
        if (!nestings.next().done) {
            let resolvedSelectors = this.resolveSelectorForNestContaining(selectorNodes, parentSelectors, container);
            while (!nestings.next().done) {
                resolvedSelectors = resolvedSelectors
                    .map(resolvedSelector => this.resolveSelectorForNestContaining(resolvedSelector.selector, parentSelectors, container))
                    .reduce((r, rs) => [...r, ...rs], []);
            }
            return resolvedSelectors;
        }
        const first = selectorNodes[0];
        if (selectors_1.isSelectorCombinator(first)) {
            return parentSelectors.map(parentSelector => new css_selector_resolver_1.ResolvedSelector([...parentSelector.selector, ...selectorNodes], container));
        }
        const comb = new ast_1.VCSSSelectorCombinator(first.node, {
            start: first.loc.start,
            end: first.loc.start,
        }, first.range[0], first.range[0], first.parent);
        comb.value = " ";
        comb.selector = " ";
        return parentSelectors.map(parentSelector => new css_selector_resolver_1.ResolvedSelector([...parentSelector.selector, comb, ...selectorNodes], container));
    }
}
exports.SCSSSelectorResolver = SCSSSelectorResolver;
